.side-nav__brand {
    background: url('logo.png') no-repeat left center !important;
    margin-left: auto;
    height: 50px;
    background-size: contain !important;
}

.side-nav__brand-text {
    display: none;
}

.side-nav {
    background-color: #21483F;
    border-right: 3px solid #DEA400;
    font-family: 'Segoe UI';
}

.side-nav__link_active {
    border-right-color: rgba(0, 0, 0, 0);
}

.side-nav__text {
    color: white;
}

.widget__title {
    font-family: 'Segoe UI';
    text-transform: none;
    font-weight: 500;
}

.button_inverse {
    color: white;
    border-color: white;
}